/*
 * Copyright 2012 M3, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package com.m3.multilane.action;

import com.m3.scalaflavor4j.Either;

/**
 * Action that will be executed asynchronously
 *
 * @param <INPUT>  input type
 * @param <OUTPUT> output type
 */
public interface Action<INPUT, OUTPUT> {

    /**
     * Getter for input
     *
     * @return input
     */
    INPUT getInput();

    /**
     * Setter for input
     *
     * @param input input
     */
    void setInput(INPUT input);

    /**
     * Getter for timeout millis
     *
     * @return timeout millis
     */
    Integer getTimeoutMillis();

    /**
     * Setter for timeout millis
     *
     * @param millis timeout millis
     */
    void setTimeoutMillis(Integer millis);

    /**
     * Apply this action synchronously
     *
     * @return result as an either
     */
    Either<Throwable, OUTPUT> apply();

}